/**
 * This class provides access to device accelerometer data.
 * @constructor
 */
function Accelerometer() {
    /**
     * The last known acceleration.
     */
    this.lastAcceleration = null;
}

/**
 * Asynchronously aquires the current acceleration.
 * @param {Function} successCallback The function to call when the acceleration
 * data is available
 * @param {Function} errorCallback The function to call when there is an error 
 * getting the acceleration data.
 * @param {AccelerationOptions} options The options for getting the accelerometer data
 * such as timeout.
 */
__PG_ACCELEROMETER_CALLBACK_USER = null;
__PG_ACCELEROMETER_CALLBACK = function(x,y,z)
{
    var accel =  { x:x, y:y, z:z };
    __PG_ACCELEROMETER_CALLBACK_USER(accel);
    Accelerometer.lastAcceleration = accel;
}

Accelerometer.prototype.getCurrentAcceleration = function(successCallback, errorCallback, options) {
    if (typeof successCallback == "function") {
        _NativeAccelerometer.get();
        __PG_ACCELEROMETER_CALLBACK_USER = successCallback;
    }
}

/**
 * Asynchronously aquires the acceleration repeatedly at a given interval.
 * @param {Function} successCallback The function to call each time the acceleration
 * data is available
 * @param {Function} errorCallback The function to call when there is an error 
 * getting the acceleration data.
 * @param {AccelerationOptions} options The options for getting the accelerometer data
 * such as timeout.
 */

Accelerometer.prototype.watchAcceleration = function(successCallback, errorCallback, options) {
    this.getCurrentAcceleration(successCallback, errorCallback, options);
    // TODO: add the interval id to a list so we can clear all watches
     var frequency = (options != undefined)? options.frequency : 10000;
    return setInterval(function() {
        navigator.accelerometer.getCurrentAcceleration(successCallback, errorCallback, options);
    }, frequency);
}

/**
 * Clears the specified accelerometer watch.
 * @param {String} watchId The ID of the watch returned from #watchAcceleration.
 */
Accelerometer.prototype.clearWatch = function(watchId) {
    clearInterval(watchId);
}

if (typeof navigator.accelerometer == "undefined") navigator.accelerometer = new Accelerometer();
